@@ -123,3 +123,10 @@ let remove () =
         glib_stop ()
     | State_lwt_into_glib engine ->
         Lwt_engine.set engine
+
+(* +-----------------------------------------------------------------+
+   | Misc                                                            |
+   +-----------------------------------------------------------------+ *)
+
+external iter : bool -> unit = "lwt_glib_iter"
+external wakeup : unit -> unit = "lwt_glib_wakeup"
