@@ -37,16 +37,17 @@ let state = ref Normal
 let raw_count = ref 0
 
 let get_attr () =
-  try
-    Some(Unix.tcgetattr Unix.stdin)
-  with
-      _ -> None
+  try_lwt
+    lwt attr = Lwt_unix.tcgetattr Lwt_unix.stdin in
+    return (Some attr)
+  with _ ->
+    return None
 
 let set_attr mode =
-  try
-    Unix.tcsetattr Unix.stdin Unix.TCSAFLUSH mode
-  with
-      _ -> ()
+  try_lwt
+    Lwt_unix.tcsetattr Lwt_unix.stdin Unix.TCSAFLUSH mode
+  with _ ->
+    return ()
 
 let drawing_mode = ref false
 
@@ -106,8 +107,7 @@ let cleanup () =
     | Normal ->
         return ()
     | Raw saved_attr ->
-        set_attr saved_attr;
-        return ()
+        set_attr saved_attr
 
 let () = Lwt_main.at_exit cleanup
 
@@ -123,8 +123,7 @@ let leave_raw_mode () =
           assert false
       | Raw attr ->
           state := Normal;
-          set_attr attr;
-          return ()
+          set_attr attr
   else
     return ()
 
@@ -134,11 +133,11 @@ let with_raw_mode f =
         incr raw_count;
         finalize f leave_raw_mode
     | Normal ->
-        match get_attr () with
+        get_attr () >>= function
           | Some attr ->
               incr raw_count;
               state := Raw attr;
-              set_attr {
+              lwt () = set_attr {
                 attr with
                   (* Inspired from Python-3.0/Lib/tty.py: *)
                   Unix.c_brkint = false;
@@ -152,10 +151,10 @@ let with_raw_mode f =
                   Unix.c_isig = false;
                   Unix.c_vmin = 1;
                   Unix.c_vtime = 0
-              };
+              } in
               try_lwt f () finally leave_raw_mode ()
           | None ->
-              fail (Failure "Lwt_term.with_raw_mode: input is not a tty")
+              raise_lwt (Failure "Lwt_term.with_raw_mode: input is not a tty")
 
 (* +-----------------------------------------------------------------+
    | Terminal informations                                           |
@@ -166,21 +165,42 @@ type size = {
   columns : int;
 }
 
-external get_size : unit -> size = "lwt_unix_term_size"
-external sigwinch : unit -> int = "lwt_unix_sigwinch"
+external get_size : Unix.file_descr -> size = "lwt_text_term_size"
+
+#if windows
+
+let size =
+  React.S.const
+    (try
+       get_size Unix.stdout
+     with Unix.Unix_error _ ->
+       { columns = 80; lines = 25 })
+
+#else
+
+external sigwinch : unit -> int = "lwt_text_sigwinch"
+let sigwinch = sigwinch ()
 
 let sigwinch_event =
-  let event, push = React.E.create () in
-  try
-    let _ = Lwt_unix.on_signal (sigwinch ()) push in
-    event
-  with Unix.Unix_error _ | Invalid_argument _ | Sys_error _ ->
+  if sigwinch = 0 then
     React.E.never
+  else
+    try
+      let event, push = React.E.create () in
+      let _ = Lwt_unix.on_signal sigwinch push in
+      event
+    with Unix.Unix_error _ | Invalid_argument _ | Sys_error _ ->
+      React.E.never
 
 let size =
   React.S.hold
-    (try get_size () with _ -> { columns = 80; lines = 25 })
-    (React.E.map (fun _ -> get_size ()) sigwinch_event)
+    (try
+       get_size Unix.stdin
+     with Unix.Unix_error _ ->
+       { columns = 80; lines = 25 })
+    (React.E.map (fun _ -> get_size Unix.stdin) sigwinch_event)
+
+#endif
 
 let columns = React.S.map (fun { columns = c } -> c) size
 let lines = React.S.map (fun { lines = l } -> l) size
@@ -200,11 +220,11 @@ let parse_escape st =
       | Sleep ->
           (* If the rest is not immediatly available, conclude that
              this is not an escape sequence but just the escape key: *)
-          fail Exit_sequence
+          raise_lwt Exit_sequence
       | Fail exn ->
-          fail exn
+          raise_lwt exn
       | Return None ->
-          fail Exit_sequence
+          raise_lwt Exit_sequence
       | Return(Some ch) ->
           (* Is it an ascii character ? *)
           if String.length ch = 1 then begin
@@ -212,7 +232,7 @@ let parse_escape st =
             return ch.[0]
           end else
             (* If it is not, then this is not an escape sequence: *)
-            fail Exit_sequence
+            raise_lwt Exit_sequence
   in
 
   (* Sometimes sequences starts with several escape characters: *)
@@ -475,7 +495,45 @@ let rec add_int buf = function
       add_int buf (n / 10);
       Buffer.add_char buf (Char.unsafe_chr (48 + (n mod 10)))
 
-let render m =
+let render_char buf oc pt last_style =
+  lwt () =
+    if pt.style <> last_style then begin
+      Buffer.clear buf;
+      Buffer.add_string buf "\027[0";
+      let mode n = function
+        | true ->
+            Buffer.add_char buf ';';
+            add_int buf n
+        | false ->
+            ()
+      and color f col =
+        if col = default then
+          ()
+        else if col < 8 then begin
+          Buffer.add_char buf ';';
+          add_int buf (f col)
+        end else begin
+          Buffer.add_char buf ';';
+          add_int buf (f 8);
+          Buffer.add_string buf ";5;";
+          add_int buf col;
+        end
+      in
+      mode Codes.bold pt.style.bold;
+      mode Codes.underlined pt.style.underlined;
+      mode Codes.blink pt.style.blink;
+      mode Codes.inverse pt.style.inverse;
+      mode Codes.hidden pt.style.hidden;
+      color Codes.foreground pt.style.foreground;
+      color Codes.background pt.style.background;
+      Buffer.add_char buf 'm';
+      write oc (Buffer.contents buf)
+    end else
+      return ()
+  in
+  write_char oc pt.char
+
+let render_update old m =
   let buf = Buffer.create 16 in
   Lwt_text.atomic begin fun oc ->
     let rec loop_y y last_style =
@@ -483,47 +541,27 @@ let render m =
         let rec loop_x x last_style =
           if x < Array.length m.(y) then
             let pt = m.(y).(x) in
-            lwt () =
-              if pt.style <> last_style then begin
-                Buffer.clear buf;
-                Buffer.add_string buf "\027[0";
-                let mode n = function
-                  | true ->
-                      Buffer.add_char buf ';';
-                      add_int buf n
-                  | false ->
-                      ()
-                and color f col =
-                  if col = default then
-                    ()
-                  else if col < 8 then begin
-                    Buffer.add_char buf ';';
-                    add_int buf (f col)
-                  end else begin
-                    Buffer.add_char buf ';';
-                    add_int buf (f 8);
-                    Buffer.add_string buf ";5;";
-                    add_int buf col;
-                  end
-                in
-                mode Codes.bold pt.style.bold;
-                mode Codes.underlined pt.style.underlined;
-                mode Codes.blink pt.style.blink;
-                mode Codes.inverse pt.style.inverse;
-                mode Codes.hidden pt.style.hidden;
-                color Codes.foreground pt.style.foreground;
-                color Codes.background pt.style.background;
-                Buffer.add_char buf 'm';
-                write oc (Buffer.contents buf)
-              end else
-                return ()
-            in
-            lwt () = write_char oc pt.char in
+            lwt () = render_char buf oc pt last_style in
             loop_x (x + 1) pt.style
           else
             loop_y (y + 1) last_style
         in
-        loop_x 0 last_style
+        if y < Array.length old && old.(y) = m.(y) then begin
+          if y + 1 < Array.length m then
+            lwt last_style =
+              if Array.length m.(y) > 0 then
+                let pt = m.(y).(0) in
+                lwt () = render_char buf oc pt last_style in
+                return pt.style
+              else
+                return last_style
+            in
+            lwt () = write oc "\r\n" in
+            loop_y (y + 1) last_style
+          else
+            return ()
+        end else
+          loop_x 0 last_style
       else
         return ()
     in
@@ -533,6 +571,8 @@ let render m =
     write oc "\027[0m"
   end stdout
 
+let render m = render_update [||] m
+
 (* +-----------------------------------------------------------------+
    | Styled text                                                     |
    +-----------------------------------------------------------------+ *)
@@ -657,28 +697,31 @@ let styled_length st =
   loop 0 st
 
 let printc st =
-  if Unix.isatty Unix.stdout then
-    atomic (fun oc -> write_styled oc st) stdout
-  else
-    write stdout (strip_styles st)
+  Lwt_unix.isatty Lwt_unix.stdout >>= function
+    | true ->
+        atomic (fun oc -> write_styled oc st) stdout
+    | false ->
+        write stdout (strip_styles st)
 
 let eprintc st =
-  if Unix.isatty Unix.stderr then
-    atomic (fun oc -> write_styled oc st) stderr
-  else
-    write stderr (strip_styles st)
+  Lwt_unix.isatty Lwt_unix.stderr >>= function
+    | true ->
+        atomic (fun oc -> write_styled oc st) stderr
+    | false ->
+        write stderr (strip_styles st)
 
 let fprintlc oc fd st =
-  if Unix.isatty fd then
-    atomic (fun oc ->
-              lwt () = write_styled oc st in
-              lwt () = write oc "\027[m" in
-              write_char oc "\n") oc
-  else
-    write_line oc (strip_styles st)
+  Lwt_unix.isatty fd >>= function
+    | true ->
+        atomic (fun oc ->
+                  lwt () = write_styled oc st in
+                  lwt () = write oc "\027[m" in
+                  write_char oc "\n") oc
+    | false ->
+        write_line oc (strip_styles st)
 
-let printlc st = fprintlc stdout Unix.stdout st
-let eprintlc st = fprintlc stderr Unix.stderr st
+let printlc st = fprintlc stdout Lwt_unix.stdout st
+let eprintlc st = fprintlc stderr Lwt_unix.stderr st
 
 (* +-----------------------------------------------------------------+
    | Drawing                                                         |
