@@ -161,6 +161,18 @@ external check_job : 'a job -> int -> bool = "lwt_unix_check_job" "noalloc"
 external cancel_job : 'a job -> unit = "lwt_unix_cancel_job" "noalloc"
     (* Cancel the thread of the given job. *)
 
+(* All running jobs. *)
+let jobs = Lwt_sequence.create ()
+
+(* Cancel all running jobs. *)
+let rec cancel_jobs () =
+  match Lwt_sequence.take_opt_l jobs with
+    | Some w -> cancel w; cancel_jobs ()
+    | None -> ()
+
+let wait_for_jobs () =
+  join (Lwt_sequence.fold_l (fun w l -> w :: l) jobs [])
+
 let execute_job ?async_method ~job ~result ~free =
   let async_method =
     match async_method with
@@ -172,61 +184,70 @@ let execute_job ?async_method ~job ~result ~free =
   in
   (* Starts the job. *)
   let job_done = start_job job async_method in
-  lwt status =
-    if job_done then
-      return None
-    else
-      (* Create the notification for asynchronous wakeup. *)
-      let id = make_notification ~once:true ignore in
-      try_lwt
-        (* Give some time to the job before we fallback to
-           asynchronous notification. *)
-        lwt () = pause () in
-        if check_job job id then begin
-          stop_notification id;
-          return None
-        end else
-          return (Some id)
-      with Canceled as exn ->
-        cancel_job job;
-        (* Free resources when the job terminates. *)
-        if check_job job id then begin
-          stop_notification id;
-          free job
-        end else
-          set_notification id (fun () -> free job);
-        raise_lwt exn
+  let w =
+    lwt status =
+      if job_done then
+        return None
+      else
+        (* Create the notification for asynchronous wakeup. *)
+        let id = make_notification ~once:true ignore in
+        try_lwt
+          (* Give some time to the job before we fallback to
+             asynchronous notification. *)
+          lwt () = pause () in
+          if check_job job id then begin
+            stop_notification id;
+            return None
+          end else
+            return (Some id)
+        with Canceled as exn ->
+          cancel_job job;
+          (* Free resources when the job terminates. *)
+          if check_job job id then begin
+            stop_notification id;
+            free job
+          end else
+            set_notification id (fun () -> free job);
+          raise_lwt exn
+    in
+    match status with
+      | None ->
+          (* The job has already terminated, read and return the result
+             immediatly. *)
+          let thread =
+            try
+              return (result job)
+            with exn ->
+              fail exn
+          in
+          free job;
+          thread
+      | Some id ->
+          (* The job has not terminated, setup the notification for the
+             asynchronous wakeup. *)
+          let waiter, wakener = task () in
+          set_notification id
+            (fun () ->
+               begin
+                 try
+                   wakeup wakener (result job);
+                 with exn ->
+                   wakeup_exn wakener exn
+               end;
+               free job);
+          on_cancel waiter
+            (fun () ->
+               cancel_job job;
+               set_notification id (fun () -> free job));
+          waiter
   in
-  match status with
-    | None ->
-        (* The job has already terminated, read and return the result
-           immediatly. *)
-        let thread =
-          try
-            return (result job)
-          with exn ->
-            fail exn
-        in
-        free job;
-        thread
-    | Some id ->
-        (* The job has not terminated, setup the notification for the
-           asynchronous wakeup. *)
-        let waiter, wakener = task () in
-        set_notification id
-          (fun () ->
-             begin
-               try
-                 wakeup wakener (result job);
-               with exn ->
-                 wakeup_exn wakener exn
-             end;
-             free job);
-        on_cancel waiter
-          (fun () ->
-             cancel_job job;
-             set_notification id (fun () -> free job));
-        waiter
+  if state w = Sleep then begin
+    (* Add the job to the sequence of all jobs. *)
+    let node = Lwt_sequence.add_l (w >> return ()) jobs in
+    (* Remove it on termination. *)
+    on_termination w (fun () -> Lwt_sequence.remove node)
+  end;
+  w
 
 (* +-----------------------------------------------------------------+
    | File descriptor wrappers                                        |
@@ -262,31 +283,30 @@ type file_descr = {
 
 #if windows
 
+external is_socket : Unix.file_descr -> bool = "lwt_unix_is_socket" "noalloc"
+
 let is_blocking ?blocking ?(set_flags=true) fd =
-    match (Unix.fstat fd).Unix.st_kind with
-      | Unix.S_SOCK -> begin
-          match blocking, set_flags with
-            | Some state, false ->
-                lazy(return state)
-            | Some true, true ->
-                Unix.clear_nonblock fd;
-                lazy(return true)
-            | Some false, true ->
-                Unix.set_nonblock fd;
-                lazy(return false)
-            | None, false ->
-                lazy(return false)
-            | None, true ->
-                Unix.set_nonblock fd;
-                lazy(return false)
-        end
-      | _ -> begin
-          match blocking with
-            | Some state ->
-                lazy(return state)
-            | None ->
-                lazy(return true)
-        end
+  if is_socket fd then
+    match blocking, set_flags with
+      | Some state, false ->
+          lazy(return state)
+      | Some true, true ->
+          Unix.clear_nonblock fd;
+          lazy(return true)
+      | Some false, true ->
+          Unix.set_nonblock fd;
+          lazy(return false)
+      | None, false ->
+          lazy(return false)
+      | None, true ->
+          Unix.set_nonblock fd;
+          lazy(return false)
+  else
+    match blocking with
+      | Some state ->
+          lazy(return state)
+      | None ->
+          lazy(return true)
 
 #else
 
@@ -1763,11 +1783,41 @@ let accept_n ch n =
   with exn ->
     return (List.rev !l, Some exn)
 
+#if windows
+
 let connect ch addr =
   (* [in_progress] tell wether connection has started but not
      terminated: *)
   let in_progress = ref false in
-  wrap_syscall Write ch begin fun _ ->
+  wrap_syscall Write ch begin fun () ->
+    if !in_progress then
+      (* Nothing works without this test and i have no idea why... *)
+      if writable ch then
+        try
+          Unix.connect ch.fd addr
+        with
+          | Unix.Unix_error (Unix.EISCONN, _, _) ->
+              (* This is the windows way of telling that the connection
+                 has completed. *)
+              ()
+      else
+        raise Retry
+    else
+      try
+        Unix.connect ch.fd addr
+      with
+        | Unix.Unix_error (Unix.EWOULDBLOCK, _, _) ->
+            in_progress := true;
+            raise Retry
+  end
+
+#else
+
+let connect ch addr =
+  (* [in_progress] tell wether connection has started but not
+     terminated: *)
+  let in_progress = ref false in
+  wrap_syscall Write ch begin fun () ->
     if !in_progress then
       (* If the connection is in progress, [getsockopt_error] tells
          wether it succceed: *)
@@ -1784,11 +1834,13 @@ let connect ch addr =
            is interrupted by a signal: *)
         Unix.connect ch.fd addr
       with
-        | Unix.Unix_error(Unix.EINPROGRESS, _, _) ->
+        | Unix.Unix_error (Unix.EINPROGRESS, _, _) ->
             in_progress := true;
             raise Retry
   end
 
+#endif
+
 let setsockopt ch opt v =
   check_descriptor ch;
   Unix.setsockopt ch.fd opt v
@@ -2340,22 +2392,42 @@ let on_signal signum handler =
   let node = Lwt_sequence.add_r handler actions in
   lazy(Lwt_sequence.remove node;
        if Lwt_sequence.is_empty actions then begin
+         signals := Signal_map.remove signum !signals;
          stop_notification notification;
          Sys.set_signal signum Sys.Signal_default
        end)
 
 let disable_signal_handler = Lazy.force
 
+let reinstall_signal_handler signum =
+  match try Some (Signal_map.find signum !signals) with Not_found -> None with
+    | Some (notification, actions) ->
+        Sys.set_signal signum (Sys.Signal_handle (fun signum -> send_notification notification))
+    | None ->
+        ()
+
 (* +-----------------------------------------------------------------+
    | Processes                                                       |
    +-----------------------------------------------------------------+ *)
 
+external reset_after_fork : unit -> unit = "lwt_unix_reset_after_fork"
+
 let fork () =
   match Unix.fork () with
     | 0 ->
+        (* Reset threading. *)
+        reset_after_fork ();
+        (* Stop the old event for notifications. *)
         Lwt_engine.stop_event !event_notifications;
         (* Reinitialise the notification system. *)
         event_notifications := Lwt_engine.on_readable (init_notification ()) handle_notifications;
+        (* Collect all pending jobs. *)
+        let l = Lwt_sequence.fold_l (fun w l -> w :: l) jobs [] in
+        (* Remove them all. *)
+        Lwt_sequence.iter_node_l Lwt_sequence.remove jobs;
+        (* And cancel them all. We yield first so that if the program
+           do an exec just after, it won't be executed. *)
+        on_termination (Lwt_main.yield ()) (fun () -> List.iter cancel l);
         0
     | pid ->
         pid
