@@ -233,7 +233,13 @@ val abort : file_descr -> exn -> unit
 val fork : unit -> int
   (** [fork ()] does the same as [Unix.fork]. You must use this
       function instead of [Unix.fork] when you want to use Lwt in the
-      child process. *)
+      child process.
+
+      Notes:
+      - in the child process all pending jobs are canceled,
+      - if you are going to use Lwt in the parent and the child, it is
+        a good idea to call {!Lwt_io.flush_all} before callling
+        {!fork} to avoid double-flush. *)
 
 type process_status =
     Unix.process_status =
@@ -627,6 +633,12 @@ val disable_signal_handler : signal_handler_id -> unit
 val signal_count : unit -> int
   (** Returns the number of registered signal handler. *)
 
+val reinstall_signal_handler : int -> unit
+  (** [reinstall_signal_handler signum] if any signal handler is
+      registered for this signal with {!on_signal}, it reinstall the
+      signal handler (with [Sys.set_signal]). This is usefull in case
+      another part of the program install another signal handler. *)
+
 (** {6 Sockets} *)
 
 type inet_addr = Unix.inet_addr
@@ -1041,6 +1053,13 @@ val execute_job :
       the result of the job, and [free] to free its associated
       resources. *)
 
+val cancel_jobs : unit -> unit
+  (** [cancel_jobs ()] make all pending jobs to fail with
+      {!Lwt.Canceled}. *)
+
+val wait_for_jobs : unit -> unit Lwt.t
+  (** Wait for all pending jobs to terminate. *)
+
 (** {6 Notifications} *)
 
 (** Lwt internally use a pipe to send notification to the main
