@@ -40,6 +40,7 @@
 
 #include <assert.h>
 #include <stdio.h>
+#include <setjmp.h>
 
 #include "lwt_config.h"
 #include "lwt_unix.h"
@@ -61,7 +62,6 @@
 #  include <grp.h>
 #  include <netdb.h>
 #  include <termios.h>
-#  include <setjmp.h>
 #  include <sched.h>
 #  include <signal.h>
 #  include <netinet/in.h>
@@ -126,7 +126,7 @@ void lwt_unix_not_available(char const *feature)
 
 CAMLprim value lwt_unix_blit_bytes_bytes(value val_buf1, value val_ofs1, value val_buf2, value val_ofs2, value val_len)
 {
-  memcpy((char*)Caml_ba_data_val(val_buf2) + Long_val(val_ofs2),
+  memmove((char*)Caml_ba_data_val(val_buf2) + Long_val(val_ofs2),
          (char*)Caml_ba_data_val(val_buf1) + Long_val(val_ofs1),
          Long_val(val_len));
   return Val_unit;
@@ -172,12 +172,200 @@ value lwt_unix_system_byte_order()
 #endif
 }
 
+/* +-----------------------------------------------------------------+
+   | Threading                                                       |
+   +-----------------------------------------------------------------+ */
+
+#if defined(LWT_ON_WINDOWS)
+
+void lwt_unix_launch_thread(void* (*start)(void*), void* data)
+{
+  HANDLE handle = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)start, data, 0, NULL);
+  if (handle) CloseHandle(handle);
+}
+
+lwt_unix_thread lwt_unix_thread_self()
+{
+  return GetCurrentThreadId();
+}
+
+int lwt_unix_thread_equal(lwt_unix_thread thread1, lwt_unix_thread thread2)
+{
+  return thread1 == thread2;
+}
+
+void lwt_unix_mutex_init(lwt_unix_mutex *mutex)
+{
+  InitializeCriticalSection(mutex);
+}
+
+void lwt_unix_mutex_destroy(lwt_unix_mutex *mutex)
+{
+  DeleteCriticalSection(mutex);
+}
+
+void lwt_unix_mutex_lock(lwt_unix_mutex *mutex)
+{
+  EnterCriticalSection(mutex);
+}
+
+void lwt_unix_mutex_unlock(lwt_unix_mutex *mutex)
+{
+  LeaveCriticalSection(mutex);
+}
+
+struct wait_list {
+  HANDLE event;
+  struct wait_list *next;
+};
+
+struct lwt_unix_condition {
+  CRITICAL_SECTION mutex;
+  struct wait_list *waiters;
+};
+
+void lwt_unix_condition_init(lwt_unix_condition *condition)
+{
+  InitializeCriticalSection(&condition->mutex);
+  condition->waiters = NULL;
+}
+
+void lwt_unix_condition_destroy(lwt_unix_condition *condition)
+{
+  DeleteCriticalSection(&condition->mutex);
+}
+
+void lwt_unix_condition_signal(lwt_unix_condition *condition)
+{
+  struct wait_list *node;
+  EnterCriticalSection(&condition->mutex);
+
+  node = condition->waiters;
+  if (node) {
+    condition->waiters = node->next;
+    SetEvent(node->event);
+  }
+  LeaveCriticalSection(&condition->mutex);
+}
+
+void lwt_unix_condition_broadcast(lwt_unix_condition *condition)
+{
+  struct wait_list *node;
+  EnterCriticalSection(&condition->mutex);
+  for (node = condition->waiters; node; node = node->next)
+    SetEvent(node->event);
+  condition->waiters = NULL;
+  LeaveCriticalSection(&condition->mutex);
+}
+
+void lwt_unix_condition_wait(lwt_unix_condition *condition, lwt_unix_mutex *mutex)
+{
+  HANDLE event;
+  struct wait_list node;
+
+  /* Create the event for the notification. */
+  node.event = CreateEvent(NULL, FALSE, FALSE, NULL);
+
+  /* Add the node to the condition. */
+  EnterCriticalSection(&condition->mutex);
+  node.next = condition->waiters;
+  condition->waiters = &node;
+  LeaveCriticalSection(&condition->mutex);
+
+  /* Release the mutex. */
+  LeaveCriticalSection(mutex);
+
+  /* Wait for a signal. */
+  WaitForSingleObject(node.event, INFINITE);
+
+  /* Re-acquire the mutex. */
+  EnterCriticalSection(mutex);
+}
+
+#else
+
+void lwt_unix_launch_thread(void* (*start)(void*), void* data)
+{
+  pthread_t thread;
+  pthread_attr_t attr;
+  int result;
+
+  pthread_attr_init(&attr);
+
+  /* The thread is created in detached state so we do not have to join
+     it when it terminates: */
+  pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
+
+  result = pthread_create(&thread, &attr, start, data);
+
+  if (result) unix_error(result, "launch_thread", Nothing);
+
+  pthread_attr_destroy (&attr);
+}
+
+lwt_unix_thread lwt_unix_thread_self()
+{
+  return pthread_self();
+}
+
+int lwt_unix_thread_equal(lwt_unix_thread thread1, lwt_unix_thread thread2)
+{
+  return pthread_equal(thread1, thread2);
+}
+
+void lwt_unix_mutex_init(lwt_unix_mutex *mutex)
+{
+  pthread_mutex_init(mutex, NULL);
+}
+
+void lwt_unix_mutex_destroy(lwt_unix_mutex *mutex)
+{
+  pthread_mutex_destroy(mutex);
+}
+
+void lwt_unix_mutex_lock(lwt_unix_mutex *mutex)
+{
+  pthread_mutex_lock(mutex);
+}
+
+void lwt_unix_mutex_unlock(lwt_unix_mutex *mutex)
+{
+  pthread_mutex_unlock(mutex);
+}
+
+void lwt_unix_condition_init(lwt_unix_condition *condition)
+{
+  pthread_cond_init(condition, NULL);
+}
+
+void lwt_unix_condition_destroy(lwt_unix_condition *condition)
+{
+  pthread_cond_destroy(condition);
+}
+
+void lwt_unix_condition_signal(lwt_unix_condition *condition)
+{
+  pthread_cond_signal(condition);
+}
+
+void lwt_unix_condition_broadcast(lwt_unix_condition *condition)
+{
+  pthread_cond_broadcast(condition);
+}
+
+void lwt_unix_condition_wait(lwt_unix_condition *condition, lwt_unix_mutex *mutex)
+{
+  pthread_cond_wait(condition, mutex);
+}
+
+#endif
+
 /* +-----------------------------------------------------------------+
    | Notifications                                                   |
    +-----------------------------------------------------------------+ */
 
 /* The mutex used to send and receive notifications. */
-static pthread_mutex_t notification_mutex;
+static lwt_unix_mutex notification_mutex;
 
 /* All pending notifications. */
 static long *notifications = NULL;
@@ -217,7 +405,7 @@ static int (*notification_recv)();
 
 static void init_notifications()
 {
-  pthread_mutex_init(&notification_mutex, NULL);
+  lwt_unix_mutex_init(&notification_mutex);
   notification_count = 4096;
   notifications = (long*)lwt_unix_malloc(notification_count * sizeof(long));
 }
@@ -234,7 +422,8 @@ static void resize_notifications()
 
 void lwt_unix_send_notification(int id)
 {
-  pthread_mutex_lock(&notification_mutex);
+  int ret;
+  lwt_unix_mutex_lock(&notification_mutex);
   if (notification_index > 0) {
     /* There is already a pending notification in the buffer, no
        need to signal the main thread. */
@@ -243,18 +432,27 @@ void lwt_unix_send_notification(int id)
   } else {
     /* There is none, notify the main thread. */
     notifications[notification_index++] = id;
+#if defined(LWT_ON_WINDOWS)
+    ret = notification_send();
+    if (ret == SOCKET_ERROR) {
+      lwt_unix_mutex_unlock(&notification_mutex);
+      win32_maperr(WSAGetLastError());
+      uerror("send_notification", Nothing);
+    }
+#else
     for (;;) {
-      int ret = notification_send();
+      ret = notification_send();
       if (ret < 0) {
         if (errno != EINTR) {
-          pthread_mutex_unlock(&notification_mutex);
+          lwt_unix_mutex_unlock(&notification_mutex);
           uerror("send_notification", Nothing);
         }
       } else
         break;
     }
+#endif
   }
-  pthread_mutex_unlock(&notification_mutex);
+  lwt_unix_mutex_unlock(&notification_mutex);
 }
 
 value lwt_unix_send_notification_stub(value id)
@@ -265,26 +463,36 @@ value lwt_unix_send_notification_stub(value id)
 
 value lwt_unix_recv_notifications()
 {
-  pthread_mutex_lock(&notification_mutex);
+  int ret, i;
+  value result;
+  lwt_unix_mutex_lock(&notification_mutex);
   /* Receive the signal. */
+#if defined(LWT_ON_WINDOWS)
+  ret = notification_recv();
+  if (ret == SOCKET_ERROR) {
+    lwt_unix_mutex_unlock(&notification_mutex);
+    win32_maperr(WSAGetLastError());
+    uerror("recv_notifications", Nothing);
+  }
+#else
   for (;;) {
-    int ret = notification_recv();
+    ret = notification_recv();
     if (ret < 0) {
       if (errno != EINTR) {
-        pthread_mutex_unlock(&notification_mutex);
+        lwt_unix_mutex_unlock(&notification_mutex);
         uerror("recv_notifications", Nothing);
       }
     } else
       break;
   }
+#endif
   /* Read all pending notifications. */
-  value result = caml_alloc_tuple(notification_index);
-  int i;
+  result = caml_alloc_tuple(notification_index);
   for (i = 0; i < notification_index; i++)
     Field(result, i) = Val_long(notifications[i]);
   /* Reset the index. */
   notification_index = 0;
-  pthread_mutex_unlock(&notification_mutex);
+  lwt_unix_mutex_unlock(&notification_mutex);
   return result;
 }
 
@@ -319,6 +527,16 @@ static int windows_notification_recv()
 
 value lwt_unix_init_notification()
 {
+  union {
+    struct sockaddr_in inaddr;
+    struct sockaddr addr;
+  } a;
+  SOCKET listener;
+  int e;
+  int addrlen = sizeof(a.inaddr);
+  int reuse = 1;
+  DWORD err;
+
   switch (notification_mode) {
   case NOTIFICATION_MODE_NOT_INITIALIZED:
     notification_mode = NOTIFICATION_MODE_NONE;
@@ -339,16 +557,6 @@ value lwt_unix_init_notification()
      sockets. The following code simulate the socketpair call of
      unix. */
 
-  union {
-    struct sockaddr_in inaddr;
-    struct sockaddr addr;
-  } a;
-  SOCKET listener;
-  int e;
-  int addrlen = sizeof(a.inaddr);
-  int reuse = 1;
-  DWORD err;
-
   socket_r = INVALID_SOCKET;
   socket_w = INVALID_SOCKET;
 
@@ -495,30 +703,6 @@ value lwt_unix_init_notification()
 
 #endif /* defined(LWT_ON_WINDOWS) */
 
-/* +-----------------------------------------------------------------+
-   | Launching a thread                                              |
-   +-----------------------------------------------------------------+ */
-
-pthread_t lwt_unix_launch_thread(void* (*start)(void*), void* data)
-{
-  pthread_t thread;
-  pthread_attr_t attr;
-
-  pthread_attr_init(&attr);
-
-  /* The thread is created in detached state so we do not have to join
-     it when it terminates: */
-  pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
-
-  int result = pthread_create(&thread, &attr, start, data);
-
-  if (result) unix_error(result, "launch_thread", Nothing);
-
-  pthread_attr_destroy (&attr);
-
-  return thread;
-}
-
 /* +-----------------------------------------------------------------+
    | Job execution                                                   |
    +-----------------------------------------------------------------+ */
@@ -528,27 +712,43 @@ static void execute_job(lwt_unix_job job)
 {
   DEBUG("executing the job");
 
+  lwt_unix_mutex_lock(&job->mutex);
+
+  /* If the job has been canceled, do nothing. */
+  if (job->state == LWT_UNIX_JOB_STATE_CANCELED) {
+    lwt_unix_mutex_unlock(&job->mutex);
+    return;
+  }
+
+  /* Mark the job as running. */
+  job->state = LWT_UNIX_JOB_STATE_RUNNING;
+
+  /* Set the thread of the job. */
+  job->thread = lwt_unix_thread_self();
+
+  lwt_unix_mutex_unlock(&job->mutex);
+
   /* Execute the job. */
   job->worker(job);
 
   DEBUG("job done");
 
-  pthread_mutex_lock(&job->mutex);
+  lwt_unix_mutex_lock(&job->mutex);
 
   DEBUG("marking the job has done");
 
   /* Job is done. If the main thread stopped until now, asynchronous
      notification is not necessary. */
-  job->done = 1;
+  job->state = LWT_UNIX_JOB_STATE_DONE;
 
   /* Send a notification if the main thread continued its execution
      before the job terminated. */
   if (job->fast == 0) {
-    pthread_mutex_unlock(&job->mutex);
+    lwt_unix_mutex_unlock(&job->mutex);
     DEBUG("notifying the main thread");
     lwt_unix_send_notification(job->notification_id);
   } else {
-    pthread_mutex_unlock(&job->mutex);
+    lwt_unix_mutex_unlock(&job->mutex);
     DEBUG("not notifying the main thread");
   }
 }
@@ -577,14 +777,14 @@ static int thread_count = 0;
 static int pool_size = 1000;
 
 /* Condition on which pool threads are waiting. */
-static pthread_cond_t pool_condition;
+static lwt_unix_condition pool_condition;
 
 /* Queue of pending jobs. It points to the last enqueued job.  */
 static lwt_unix_job pool_queue = NULL;
 
 /* The mutex which protect access to [pool_queue], [pool_condition]
    and [thread_waiting_count]. */
-static pthread_mutex_t pool_mutex;
+static lwt_unix_mutex pool_mutex;
 
 /* +-----------------------------------------------------------------+
    | Thread switching                                                |
@@ -605,7 +805,7 @@ enum main_state {
 static enum main_state main_state = STATE_RUNNING;
 
 /* The main thread. */
-static pthread_t main_thread;
+static lwt_unix_thread main_thread;
 
 /* A node in a list of stack frames. */
 struct stack_frame {
@@ -620,7 +820,7 @@ struct stack_frame {
 static struct stack_frame *blocking_call_enter = NULL;
 
 /* Mutex to protect access to [blocking_call_enter]. */
-static pthread_mutex_t blocking_call_enter_mutex;
+static lwt_unix_mutex blocking_call_enter_mutex;
 
 /* Where to go when the blocking call is done, or when it get
    scheduled. */
@@ -649,19 +849,22 @@ static int stack_allocated;
 /* Function executed on an alternative stack. */
 static void altstack_worker()
 {
+  struct stack_frame *node;
+  jmp_buf buf;
+
   if (stack_allocated == 1) return;
   stack_allocated = 1;
 
   /* The first passage is to register a new stack frame. */
-  struct stack_frame *node = lwt_unix_new(struct stack_frame);
+  node = lwt_unix_new(struct stack_frame);
 
   if (setjmp(node->checkpoint) == 0) {
 
     /* Add it to the list of available stack frames. */
-    pthread_mutex_lock(&blocking_call_enter_mutex);
+    lwt_unix_mutex_lock(&blocking_call_enter_mutex);
     node->next = blocking_call_enter;
     blocking_call_enter = node;
-    pthread_mutex_unlock(&blocking_call_enter_mutex);
+    lwt_unix_mutex_unlock(&blocking_call_enter_mutex);
 
   } else {
 
@@ -676,9 +879,9 @@ static void altstack_worker()
     DEBUG("signaling the pool condition variable");
 
     /* Maybe wakeup a worker so it can become the main thread. */
-    pthread_mutex_lock(&pool_mutex);
-    pthread_cond_signal(&pool_condition);
-    pthread_mutex_unlock(&pool_mutex);
+    lwt_unix_mutex_lock(&pool_mutex);
+    lwt_unix_condition_signal(&pool_condition);
+    lwt_unix_mutex_unlock(&pool_mutex);
 
     DEBUG("executing the blocking call");
 
@@ -687,14 +890,14 @@ static void altstack_worker()
 
     DEBUG("blocking call done");
 
-    pthread_mutex_lock(&pool_mutex);
+    lwt_unix_mutex_lock(&pool_mutex);
 
-    if (pthread_equal(main_thread, pthread_self())) {
+    if (lwt_unix_thread_equal(main_thread, lwt_unix_thread_self())) {
       /* We stayed the main thread, continue the execution
          normally. */
       main_state = STATE_RUNNING;
 
-      pthread_mutex_unlock(&pool_mutex);
+      lwt_unix_mutex_unlock(&pool_mutex);
 
       DEBUG("blocing call terminated without blocking, resuming");
 
@@ -706,21 +909,20 @@ static void altstack_worker()
       assert(become_worker != NULL);
 
       /* Take and remove the first worker checkpoint. */
-      struct stack_frame *node = become_worker;
+      node = become_worker;
       become_worker = node->next;
 
-      pthread_mutex_unlock(&pool_mutex);
+      lwt_unix_mutex_unlock(&pool_mutex);
 
       DEBUG("blocking call terminated after blocking, becoming a worker");
 
       /* Add the stack frame used for this call to the list of
          available ones. */
-      pthread_mutex_lock(&blocking_call_enter_mutex);
+      lwt_unix_mutex_lock(&blocking_call_enter_mutex);
       frame->next = blocking_call_enter;
       blocking_call_enter = frame;
       /* Release the mutex only after the jump. */
 
-      jmp_buf buf;
       memcpy(&buf, &(node->checkpoint), sizeof(jmp_buf));
       free(node);
       longjmp(buf, 1);
@@ -784,11 +986,11 @@ static void nop() {}
 void initialize_threading()
 {
   if (threading_initialized == 0) {
-    pthread_mutex_init(&pool_mutex, NULL);
-    pthread_cond_init(&pool_condition, NULL);
-    pthread_mutex_init(&blocking_call_enter_mutex, NULL);
+    lwt_unix_mutex_init(&pool_mutex);
+    lwt_unix_condition_init(&pool_condition);
+    lwt_unix_mutex_init(&blocking_call_enter_mutex);
 
-    main_thread = pthread_self();
+    main_thread = lwt_unix_thread_self();
 
 #if !defined(LWT_ON_WINDOWS)
     if (SIGRTMIN < SIGRTMAX) {
@@ -823,6 +1025,7 @@ void initialize_threading()
 static void* worker_loop(void *data)
 {
   lwt_unix_job job = (lwt_unix_job)data;
+  struct stack_frame *node;
 
   /* Execute the initial job if any. */
   if (job != NULL) execute_job(job);
@@ -830,7 +1033,7 @@ static void* worker_loop(void *data)
   while (1) {
     DEBUG("entering waiting section");
 
-    pthread_mutex_lock(&pool_mutex);
+    lwt_unix_mutex_lock(&pool_mutex);
 
     /* One more thread is waiting for work. */
     thread_waiting_count++;
@@ -839,7 +1042,7 @@ static void* worker_loop(void *data)
 
     /* Wait for something to do. */
     while (pool_queue == NULL && main_state == STATE_RUNNING)
-      pthread_cond_wait(&pool_condition, &pool_mutex);
+      lwt_unix_condition_wait(&pool_condition, &pool_mutex);
 
     DEBUG("received something to do");
 
@@ -851,12 +1054,12 @@ static void* worker_loop(void *data)
       DEBUG("\e[1;31mswitching\e[0m");
 
       /* If the main thread is blocked, we become the main thread. */
-      main_thread = pthread_self();
+      main_thread = lwt_unix_thread_self();
 
       /* The new main thread is running again. */
       main_state = STATE_RUNNING;
 
-      struct stack_frame *node = lwt_unix_new(struct stack_frame);
+      node = lwt_unix_new(struct stack_frame);
 
       /* Save the stack frame so the old main thread can become a
          worker when the blocking call terminates. */
@@ -879,7 +1082,7 @@ static void* worker_loop(void *data)
       //caml_c_thread_unregister();
 
       /* Release this mutex. It was locked before the jump. */
-      pthread_mutex_unlock(&blocking_call_enter_mutex);
+      lwt_unix_mutex_unlock(&blocking_call_enter_mutex);
 
     } else {
       DEBUG("taking a job to execute");
@@ -887,16 +1090,13 @@ static void* worker_loop(void *data)
       /* Take the first queued job. */
       job = pool_queue->next;
 
-      job->thread = pthread_self();
-      job->thread_initialized = 1;
-
       /* Remove it from the queue. */
       if (job->next == job)
         pool_queue = NULL;
       else
         pool_queue->next = job->next;
 
-      pthread_mutex_unlock(&pool_mutex);
+      lwt_unix_mutex_unlock(&pool_mutex);
 
       /* Execute the job. */
       execute_job(job);
@@ -933,14 +1133,14 @@ value lwt_unix_alloc_job(lwt_unix_job job)
 void lwt_unix_free_job(lwt_unix_job job)
 {
   if (job->async_method != LWT_UNIX_ASYNC_METHOD_NONE)
-    pthread_mutex_destroy(&job->mutex);
+    lwt_unix_mutex_destroy(&job->mutex);
   free(job);
 }
 
 CAMLprim value lwt_unix_start_job(value val_job, value val_async_method)
 {
   lwt_unix_job job = Job_val(val_job);
-
+  struct stack_frame *node;
   lwt_unix_async_method async_method = Int_val(val_async_method);
 
   /* Fallback to synchronous call if there is no worker available and
@@ -949,7 +1149,7 @@ CAMLprim value lwt_unix_start_job(value val_job, value val_async_method)
     async_method = LWT_UNIX_ASYNC_METHOD_NONE;
 
   /* Initialises job parameters. */
-  job->done = 0;
+  job->state = LWT_UNIX_JOB_STATE_PENDING;
   job->fast = 1;
   job->async_method = async_method;
 
@@ -965,14 +1165,13 @@ CAMLprim value lwt_unix_start_job(value val_job, value val_async_method)
   case LWT_UNIX_ASYNC_METHOD_DETACH:
     if (threading_initialized == 0) initialize_threading();
 
-    pthread_mutex_init(&job->mutex, NULL);
-    job->thread_initialized = 0;
+    lwt_unix_mutex_init(&job->mutex);
 
-    pthread_mutex_lock(&pool_mutex);
+    lwt_unix_mutex_lock(&pool_mutex);
     if (thread_waiting_count == 0) {
       /* Launch a new worker. */
       thread_count++;
-      pthread_mutex_unlock(&pool_mutex);
+      lwt_unix_mutex_unlock(&pool_mutex);
       lwt_unix_launch_thread(worker_loop, (void*)job);
     } else {
       /* Add the job at the end of the queue. */
@@ -985,10 +1184,10 @@ CAMLprim value lwt_unix_start_job(value val_job, value val_async_method)
         pool_queue = job;
       }
       /* Wakeup one worker. */
-      pthread_cond_signal(&pool_condition);
-      pthread_mutex_unlock(&pool_mutex);
+      lwt_unix_condition_signal(&pool_condition);
+      lwt_unix_mutex_unlock(&pool_mutex);
     }
-    return Val_bool(job->done);
+    return Val_bool(job->state == LWT_UNIX_JOB_STATE_DONE);
 
   case LWT_UNIX_ASYNC_METHOD_SWITCH:
 #if defined(LWT_ON_WINDOWS)
@@ -997,7 +1196,7 @@ CAMLprim value lwt_unix_start_job(value val_job, value val_async_method)
 
     if (threading_initialized == 0) initialize_threading();
 
-    pthread_mutex_init(&job->mutex, NULL);
+    lwt_unix_mutex_init(&job->mutex);
     job->thread = main_thread;
 
     /* Ensures there is at least one thread that can become the main
@@ -1013,11 +1212,11 @@ CAMLprim value lwt_unix_start_job(value val_job, value val_async_method)
 
     /* Take and remove the first available stack frame for system
        calls. */
-    pthread_mutex_lock(&blocking_call_enter_mutex);
+    lwt_unix_mutex_lock(&blocking_call_enter_mutex);
     assert(blocking_call_enter != NULL);
-    struct stack_frame *node = blocking_call_enter;
+    node = blocking_call_enter;
     blocking_call_enter = node->next;
-    pthread_mutex_unlock(&blocking_call_enter_mutex);
+    lwt_unix_mutex_unlock(&blocking_call_enter_mutex);
 
     /* Save the stack frame to leave the blocking call. */
     switch (setjmp(blocking_call_leave)) {
@@ -1039,21 +1238,21 @@ CAMLprim value lwt_unix_start_job(value val_job, value val_async_method)
 
       /* Re-add the stack frame used for the call to the list of
          available ones. */
-      pthread_mutex_lock(&blocking_call_enter_mutex);
+      lwt_unix_mutex_lock(&blocking_call_enter_mutex);
       node->next = blocking_call_enter;
       blocking_call_enter = node;
-      pthread_mutex_unlock(&blocking_call_enter_mutex);
+      lwt_unix_mutex_unlock(&blocking_call_enter_mutex);
       return Val_true;
 
     case CALL_SCHEDULED:
       DEBUG("resuming after being scheduled");
 
       /* This mutex was locked before we did the jump. */
-      pthread_mutex_unlock(&pool_mutex);
+      lwt_unix_mutex_unlock(&pool_mutex);
 
       /* This thread is now running caml code. */
       //caml_c_thread_register();
-      return Val_bool(job->done);
+      return Val_bool(job->state == LWT_UNIX_JOB_STATE_DONE);
     }
   }
 
@@ -1063,6 +1262,7 @@ CAMLprim value lwt_unix_start_job(value val_job, value val_async_method)
 CAMLprim value lwt_unix_check_job(value val_job, value val_notification_id)
 {
   lwt_unix_job job = Job_val(val_job);
+  value result;
 
   DEBUG("checking job");
 
@@ -1073,13 +1273,13 @@ CAMLprim value lwt_unix_check_job(value val_job, value val_notification_id)
 
   case LWT_UNIX_ASYNC_METHOD_DETACH:
   case LWT_UNIX_ASYNC_METHOD_SWITCH:
-    pthread_mutex_lock(&job->mutex);
+    lwt_unix_mutex_lock(&job->mutex);
     /* We are not waiting anymore. */
     job->fast = 0;
     /* Set the notification id for asynchronous wakeup. */
     job->notification_id = Int_val(val_notification_id);
-    value result = Val_bool(job->done);
-    pthread_mutex_unlock(&job->mutex);
+    result = Val_bool(job->state == LWT_UNIX_JOB_STATE_DONE);
+    lwt_unix_mutex_unlock(&job->mutex);
 
     DEBUG("job done: %d", Int_val(result));
 
@@ -1101,24 +1301,56 @@ CAMLprim value lwt_unix_cancel_job(value val_job)
     break;
 
   case LWT_UNIX_ASYNC_METHOD_DETACH:
-    while (job->thread_initialized == 0) {
-      struct timeval tv;
-      tv.tv_sec = 0;
-      tv.tv_usec = 100000;
-      select(0, NULL, NULL, NULL, &tv);
-    };
-
   case LWT_UNIX_ASYNC_METHOD_SWITCH:
-    pthread_mutex_lock(&job->mutex);
-    if (job->done == 0 && signal_kill_thread >= 0)
-      pthread_kill(job->thread, signal_kill_thread);
-    pthread_mutex_unlock(&job->mutex);
+    lwt_unix_mutex_lock(&job->mutex);
+    switch (job->state) {
+    case LWT_UNIX_JOB_STATE_PENDING:
+      /* The job has not yet started, mark it as canceled. */
+      job->state = LWT_UNIX_JOB_STATE_CANCELED;
+      break;
+
+    case LWT_UNIX_JOB_STATE_RUNNING:
+#if !defined(LWT_ON_WINDOWS)
+      /* The job is running, kill it. */
+      if (signal_kill_thread >= 0)
+        pthread_kill(job->thread, signal_kill_thread);
+#endif
+      break;
+
+    case LWT_UNIX_JOB_STATE_DONE:
+      /* The job is done, do nothing. */
+      break;
+
+    case LWT_UNIX_JOB_STATE_CANCELED:
+      /* Not possible. */
+      break;
+    }
+    lwt_unix_mutex_unlock(&job->mutex);
     break;
   }
 
   return Val_unit;
 }
 
+CAMLprim value lwt_unix_reset_after_fork()
+{
+  if (threading_initialized) {
+    /* Reset the main thread. */
+    main_thread = lwt_unix_thread_self ();
+
+    /* There is no more waiting threads. */
+    thread_waiting_count = 0;
+
+    /* There is no more threads. */
+    thread_count = 0;
+
+    /* Empty the queue. */
+    pool_queue = NULL;
+  }
+
+  return Val_unit;
+}
+
 /* +-----------------------------------------------------------------+
    | Statistics and control                                          |
    +-----------------------------------------------------------------+ */
