@@ -1,3 +1,25 @@
+(* Lightweight thread library for Objective Caml
+ * http://www.ocsigen.org/lwt
+ * Module Test_lwt
+ * Copyright (C) 2010 Jérémie Dimino, Pierre Chambart
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation, with linking exceptions;
+ * either version 2.1 of the License, or (at your option) any later
+ * version. See COPYING file for details.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
+ *)
+
 open Test
 open Lwt
 
@@ -485,6 +507,23 @@ let suite = suite "lwt" [
        cancel t;
        return (state t = Fail Canceled));
 
+  test "bind/cancel 3"
+    (fun () ->
+       let waiter1, wakener1 = wait () in
+       let waiter2, wakener2 = wait () in
+       let t =
+         lwt () = waiter1 in
+         try_lwt
+           lwt () = waiter2 in
+           fst (task ())
+         with Canceled ->
+           return true
+       in
+       wakeup wakener1 ();
+       wakeup wakener2 ();
+       cancel t;
+       return (state t = Return true));
+
   test "data 1"
     (fun () ->
        with_value key (Some 1)
